@@ -11,6 +11,7 @@ from loki import Module, Sourcefile, Subroutine
 from loki.analyse import (
     dataflow_analysis_attached, read_after_write_vars, loop_carried_dependencies
 )
+from loki.analyse.analyse_dataflow import DataflowAnalysisAttacher, DataflowAnalysisDetacher
 from loki.backend import fgen
 from loki.expression import symbols as sym
 from loki.frontend import available_frontends, OMNI
@@ -169,16 +170,18 @@ end subroutine analyse_read_after_write_vars
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_read_after_write_vars_conditionals(frontend):
+@pytest.mark.parametrize('include_literal_kinds', [True, False])
+def test_read_after_write_vars_conditionals(frontend, include_literal_kinds):
     fcode = """
 subroutine analyse_read_after_write_vars_conditionals(a, b, c, d, e, f)
+  use iso_fortran_env, only : int32
   integer, intent(in) :: a
   integer, intent(out) :: b, c, d, e, f
 
   b = 1
   d = 0
 !$loki A
-  if (a < 3) then
+  if (a < 3_int32) then
     d = b
 !$loki B
   endif
@@ -209,9 +212,17 @@ end subroutine analyse_read_after_write_vars_conditionals
     pragmas = FindNodes(ir.Pragma).visit(routine.body)
     assert len(pragmas) == len(vars_at_inspection_node)
 
-    with dataflow_analysis_attached(routine):
-        for pragma in pragmas:
-            assert read_after_write_vars(routine.body, pragma) == vars_at_inspection_node[pragma.content]
+    # We skip the context manager here to test the "include_literal_kinds" option
+    DataflowAnalysisAttacher(include_literal_kinds=include_literal_kinds).visit(routine.body)
+
+    if include_literal_kinds:
+        assert 'int32' in routine.body.uses_symbols
+    else:
+        assert not 'int32' in routine.body.uses_symbols
+    for pragma in pragmas:
+        assert read_after_write_vars(routine.body, pragma) == vars_at_inspection_node[pragma.content]
+
+    DataflowAnalysisDetacher().visit(routine.body)
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -418,6 +429,7 @@ end subroutine test
     routine = Subroutine.from_source(fcode, frontend=frontend)
     with dataflow_analysis_attached(routine):
         assert 'real64' in routine.body.uses_symbols
+        assert 'real64' in routine.spec.uses_symbols
         assert 'real64' not in routine.body.defines_symbols
         assert 'a' in routine.body.defines_symbols
         assert 'a' not in routine.body.uses_symbols
@@ -570,17 +582,19 @@ end subroutine test
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_analyse_maskedstatement(frontend):
+@pytest.mark.parametrize('include_literal_kinds', [True, False])
+def test_analyse_maskedstatement(frontend, include_literal_kinds):
     fcode = """
 subroutine masked_statements(n, mask, vec1, vec2)
+  use iso_fortran_env, only : int32
   integer, intent(in) :: n
   integer, intent(in), dimension(n) :: mask
   real, intent(out), dimension(n) :: vec1,vec2
 
-  where (mask(:) < -5)
+  where (mask(:) < -5_int32)
     vec1(:) = -5.0
     vec1(:) = vec1(:) -5.0
-  elsewhere (mask(:) > 5)
+  elsewhere (mask(:) > 5_int32)
     vec1(:) =  5.0
   elsewhere
     vec1(:) = 0.0
@@ -592,11 +606,21 @@ end subroutine masked_statements
     routine = Subroutine.from_source(fcode, frontend=frontend)
     mask = FindNodes(ir.MaskedStatement).visit(routine.body)[0]
     num_bodies = len(mask.bodies)
-    with dataflow_analysis_attached(routine):
+
+    # We skip the context manager here to test the "include_literal_kinds" option
+    DataflowAnalysisAttacher(include_literal_kinds=include_literal_kinds).visit(routine.body)
+
+    if include_literal_kinds:
+        assert len(mask.uses_symbols) == 2
+        assert 'int32' in mask.uses_symbols
+    else:
         assert len(mask.uses_symbols) == 1
-        assert len(mask.defines_symbols) == 1
-        assert 'mask' in mask.uses_symbols
-        assert 'vec1' in mask.defines_symbols
+        assert not 'int32' in mask.uses_symbols
+    assert len(mask.defines_symbols) == 1
+    assert 'mask' in mask.uses_symbols
+    assert 'vec1' in mask.defines_symbols
+
+    DataflowAnalysisDetacher().visit(routine.body)
 
     assert len(mask.bodies) == num_bodies
 
