@@ -14,7 +14,7 @@ import numpy as np
 
 from loki import Subroutine
 from loki.backend import pygen
-from loki.build import jit_compile, clean_test
+from loki.jit_build import jit_compile, clean_test
 from loki.frontend import available_frontends, OMNI
 from loki.transformations.transpile import FortranPythonTransformation
 
@@ -152,9 +152,9 @@ end subroutine pygen_arguments
     array = np.zeros(shape=(n,), order='F')
     array_io = np.zeros(shape=(n,), order='F') + 3.
     # To do scalar inout we allocate data in single-element arrays
-    a_io = np.zeros(shape=(1,), order='F', dtype=np.int32) + 1
-    b_io = np.zeros(shape=(1,), order='F', dtype=np.float32) + 2.
-    c_io = np.zeros(shape=(1,), order='F', dtype=np.float64) + 3.
+    a_io = np.array(1)
+    b_io = np.array(2.)
+    c_io = np.array(3.)
 
     # Generate reference code, compile run and verify
     routine = Subroutine.from_source(fcode, frontend=frontend)
@@ -164,7 +164,7 @@ end subroutine pygen_arguments
 
     assert np.all(array == 3.) and array.size == n
     assert np.all(array_io == 6.)
-    assert a_io[0] == 3. and np.isclose(b_io[0], 5.2) and np.isclose(c_io[0], 7.1)
+    assert a_io == 3 and np.isclose(b_io, 5.2) and np.isclose(c_io, 7.1)
     assert a == 8 and np.isclose(b, 3.2) and np.isclose(c, 4.1)
 
     # Rename routine to avoid problems with module import caching
@@ -178,14 +178,14 @@ end subroutine pygen_arguments
 
     array = np.zeros(shape=(n,), order='F')
     array_io = np.zeros(shape=(n,), order='F') + 3.
-    a_io = np.zeros(shape=(1,), order='F', dtype=np.int32) + 1
-    b_io = np.zeros(shape=(1,), order='F', dtype=np.float32) + 2.
-    c_io = np.zeros(shape=(1,), order='F', dtype=np.float64) + 3.
+    a_io = np.array(1)
+    b_io = np.array(2.)
+    c_io = np.array(3.)
     a, b, c, a_io, b_io, c_io = func(n, array, array_io, a_io, b_io, c_io)
 
     assert np.all(array == 3.) and array.size == n
     assert np.all(array_io == 6.)
-    assert a_io[0] == 3. and np.isclose(b_io[0], 5.2) and np.isclose(c_io[0], 7.1)
+    assert a_io == 3. and np.isclose(b_io, 5.2) and np.isclose(c_io, 7.1)
     assert a == 8 and np.isclose(b, 3.2) and np.isclose(c, 4.1)
 
     clean_test(filepath)
@@ -336,14 +336,20 @@ subroutine pygen_loop_indices(n, idx, mask1, mask2, mask3)
   do i=1, n
      if (i < idx) then
         mask1(i) = 1
-     end if
-
-     if (i == idx) then
+     elseif (i == idx) then
         mask1(i) = 2
+     else
+        mask1(i) = 3
      end if
 
      mask2(i) = i
   end do
+
+  i = 1
+  do while (i <= idx)
+    mask3(i) = 2.0
+    i = i + 1
+  end do
   mask3(n) = 3.0
 end subroutine pygen_loop_indices
 """
@@ -363,9 +369,9 @@ end subroutine pygen_loop_indices
     function(n=n, idx=fidx, mask1=mask1, mask2=mask2, mask3=mask3)
     assert np.all(mask1[:cidx-1] == 1)
     assert mask1[cidx] == 2
-    assert np.all(mask1[cidx+1:] == 0)
+    assert np.all(mask1[cidx+1:] == 3)
     assert np.all(mask2 == np.arange(n, dtype=np.int32) + 1)
-    assert np.all(mask3[:-1] == 0.)
+    assert np.all(mask3[:fidx] == 2.)
     assert mask3[-1] == 3.
 
     # Rename routine to avoid problems with module import caching
@@ -383,9 +389,9 @@ end subroutine pygen_loop_indices
     func(n=n, idx=fidx, mask1=mask1, mask2=mask2, mask3=mask3)
     assert np.all(mask1[:cidx-1] == 1)
     assert mask1[cidx] == 2
-    assert np.all(mask1[cidx+1:] == 0)
+    assert np.all(mask1[cidx+1:] == 3)
     assert np.all(mask2 == np.arange(n, dtype=np.int32) + 1)
-    assert np.all(mask3[:-1] == 0.)
+    assert np.all(mask3[:fidx] == 2.)
     assert mask3[-1] == 3.
 
     clean_test(filepath)
