@@ -238,6 +238,14 @@ class Item(ItemConfig):
         """
         return self.ir
 
+    transformation_ir = scope_ir
+    """
+    Return the nearest :any:`Scope` IR node that corresponds to a suitable
+    transformation entry point, i.e., is a :any:`ProgramUnit` or :any:`Sourcefile`.
+
+    For most but not all item types this is equivalent to ``scope_ir``.
+    """
+
     def _parser_classes_from_item_type_names(self, item_type_names):
         """
         Helper method that queries the :attr:`Item._parser_class` of all
@@ -698,7 +706,7 @@ class ProcedureItem(Item):
     _parser_class = RegexParserClass.ProgramUnitClass
     _depends_class = (
         RegexParserClass.ImportClass | RegexParserClass.InterfaceClass | RegexParserClass.TypeDefClass |
-        RegexParserClass.DeclarationClass | RegexParserClass.CallClass
+        RegexParserClass.DeclarationClass | RegexParserClass.CallClass | RegexParserClass.PragmaClass
     )
 
     @property
@@ -792,6 +800,12 @@ class TypeDefItem(Item):
             if isinstance(decl, ProcedureDeclaration)
         )
 
+    transformation_ir = Item.scope
+    """
+    The transformation entry point for TypeDefItem is the scope in which the
+    typedef is declared.
+    """
+
 
 class InterfaceItem(Item):
     """
@@ -922,6 +936,12 @@ class ProcedureBindingItem(Item):
         proc_name = f'{symbol_name}%{name_parts[-1]}'
         return as_tuple(ProcedureSymbol(name=proc_name, parent=symbol, scope=symbol.scope))
 
+    transformation_ir = Item.scope
+    """
+    The transformation entry point is the scope in which the typedef is declared
+    to which this procedure binding is attached.
+    """
+
 
 class ExternalItem(Item):
     """
@@ -939,10 +959,14 @@ class ExternalItem(Item):
     ----------
     origin_cls :
         The subclass of :any:`Item` this item represents.
+    is_generated : bool
+        A flag that indicates whether this item is expected to be generated by the
+        build.
     """
 
-    def __init__(self, name, source, config=None, origin_cls=None):
+    def __init__(self, name, source, config=None, origin_cls=None, is_generated=False):
         self.origin_cls = origin_cls
+        self.is_generated = is_generated
         super().__init__(name, source, config)
 
     @property
