@@ -9,8 +9,11 @@ Expression layer of the two-level Loki IR based on `Pymbolic
 <https://github.com/inducer/pymbolic>`_.
 """
 
-from loki.expression.symbols import *  # noqa
-from loki.expression.operations import *  # noqa
+from loki.expression.evaluation import *  # noqa
+from loki.expression.literals import *  # noqa
 from loki.expression.mappers import *  # noqa
-from loki.expression.symbolic import *  # noqa
+from loki.expression.mixins import *  # noqa
+from loki.expression.operations import *  # noqa
 from loki.expression.parser import *  # noqa
+from loki.expression.symbolic import *  # noqa
+from loki.expression.symbols import *  # noqa
