@@ -305,23 +305,25 @@ class OMNI2IR(GenericVisitor):
         body = tuple(self.visit(c, **kwargs) for c in o)
         return ir.Interface(body=body, abstract=abstract, spec=spec, source=kwargs['source'])
 
-    def _create_Subroutine_object(self, o, scope, symbol_map):
+    def _create_Procedure_object(self, o, scope, symbol_map):
         """Helper method to instantiate a Subroutine object"""
+        from loki.function import Function  # pylint: disable=import-outside-toplevel,cyclic-import
         from loki.subroutine import Subroutine  # pylint: disable=import-outside-toplevel,cyclic-import
+
         assert o.tag in ('FfunctionDefinition', 'FfunctionDecl')
         name = o.find('name').text
 
         # Check if the Subroutine node has been created before by looking it up in the scope
-        routine = None
+        procedure = None
         if scope is not None and name in scope.symbol_attrs:
             proc_type = scope.symbol_attrs[name]  # Look-up only in current scope!
             if proc_type and proc_type.dtype.procedure != BasicType.DEFERRED:
-                routine = proc_type.dtype.procedure
-                if not routine._incomplete:
+                procedure = proc_type.dtype.procedure
+                if not procedure._incomplete:
                     # We return the existing object right away, unless it exists from a
                     # previous incomplete parse for which we have to make sure we get a
                     # full parse first
-                    return routine
+                    return procedure
 
         # Return type and dummy args
         ftype = self.type_map[o.find('name').attrib['type']]
@@ -342,21 +344,34 @@ class OMNI2IR(GenericVisitor):
         result = ftype.attrib.get('result_name')
 
         # Instantiate the object
-        if routine is None:
-            routine = Subroutine(
-                name=name, args=args, prefix=prefix, bind=None,
-                result_name=result, is_function=is_function, parent=scope,
-                ast=o, source=self.get_source(o)
-            )
+        if is_function:
+            if procedure is None:
+                procedure = Function(
+                    name=name, args=args, prefix=prefix, bind=None,
+                    result_name=result, parent=scope, ast=o, source=self.get_source(o)
+                )
+            else:
+                procedure.__initialize__(
+                    name=name, args=args, docstring=procedure.docstring, spec=procedure.spec,
+                    body=procedure.body, contains=procedure.contains, prefix=prefix, bind=None,
+                    result_name=result, ast=o, source=self.get_source(o),
+                    incomplete=procedure._incomplete
+                )
         else:
-            routine.__initialize__(
-                name=name, args=args, docstring=routine.docstring, spec=routine.spec,
-                body=routine.body, contains=routine.contains, prefix=prefix, bind=None,
-                result_name=result, is_function=is_function, ast=o,
-                source=self.get_source(o), incomplete=routine._incomplete
-            )
+            if procedure is None:
+                procedure = Subroutine(
+                    name=name, args=args, prefix=prefix, bind=None,
+                    parent=scope, ast=o, source=self.get_source(o)
+                )
+            else:
+                procedure.__initialize__(
+                    name=name, args=args, docstring=procedure.docstring, spec=procedure.spec,
+                    body=procedure.body, contains=procedure.contains, prefix=prefix, bind=None,
+                    ast=o, source=self.get_source(o),
+                    incomplete=procedure._incomplete
+                )
 
-        return routine
+        return procedure
 
     def visit_FfunctionDefinition(self, o, **kwargs):
         # Update the symbol map with local entries
@@ -364,7 +379,7 @@ class OMNI2IR(GenericVisitor):
         kwargs['symbol_map'].update({s.attrib['type']: s for s in o.find('symbols')})
 
         # Instantiate the object
-        routine = self._create_Subroutine_object(o, kwargs['scope'], kwargs['symbol_map'])
+        routine = self._create_Procedure_object(o, kwargs['scope'], kwargs['symbol_map'])
         if routine is None:
             return None
         kwargs['scope'] = routine
@@ -422,17 +437,19 @@ class OMNI2IR(GenericVisitor):
         # Finally, call the subroutine constructor on the object again to register all
         # bits and pieces in place and rescope all symbols
         # pylint: disable=unnecessary-dunder-call
-        routine.__initialize__(
-            name=routine.name, args=routine._dummies, docstring=docstring, spec=spec,
-            body=body, contains=contains, ast=o, prefix=routine.prefix,
-            bind=routine.bind, result_name=routine.result_name,
-            is_function=routine.is_function, rescope_symbols=True,
-            source=routine.source, incomplete=False
-        )
-
-        # For deferred array dimensions on allocatables, we infer the conceptual
-        # dimension by finding any `allocate(var(<dims>))` statements.
-        routine._infer_allocatable_shapes()
+        if routine.is_function:
+            routine.__initialize__(
+                name=routine.name, args=routine._dummies, docstring=docstring, spec=spec,
+                body=body, contains=contains, ast=o, prefix=routine.prefix,
+                bind=routine.bind, result_name=routine.result_name,
+                rescope_symbols=True, source=routine.source, incomplete=False
+            )
+        else:
+            routine.__initialize__(
+                name=routine.name, args=routine._dummies, docstring=docstring, spec=spec,
+                body=body, contains=contains, ast=o, prefix=routine.prefix,
+                bind=routine.bind, rescope_symbols=True, source=routine.source, incomplete=False
+            )
 
         # Update array shapes with Loki dimension pragmas
         with pragmas_attached(routine, ir.VariableDeclaration):
@@ -488,7 +505,7 @@ class OMNI2IR(GenericVisitor):
             # I know, it's not pretty but alternatively we could hand down this array as part of
             # kwargs but that feels like carrying around a lot of bulk, too.
             contains = [
-                self._create_Subroutine_object(member_ast, kwargs['scope'], kwargs['symbol_map'])
+                self._create_Procedure_object(member_ast, kwargs['scope'], kwargs['symbol_map'])
                 for member_ast in contains_ast.findall('FfunctionDefinition')
             ]
 
@@ -655,7 +672,7 @@ class OMNI2IR(GenericVisitor):
             body += [ir.Intrinsic('SEQUENCE')]
 
         # Build the list of derived type members and individual body for each
-        if struct_type.find('symbols'):
+        if struct_type.find('symbols') is not None:
             variables = self.visit(struct_type.find('symbols'), **kwargs)
             for v in variables:
                 if isinstance(v.type.dtype, ProcedureType):
@@ -668,7 +685,7 @@ class OMNI2IR(GenericVisitor):
                 else:
                     body += [ir.VariableDeclaration(symbols=(v,))]
 
-        if struct_type.find('typeBoundProcedures'):
+        if struct_type.find('typeBoundProcedures') is not None:
             # See if components are marked private
             body += [ir.Intrinsic('CONTAINS')]
             if struct_type.attrib.get('is_internal_private') == 'true':
@@ -730,7 +747,7 @@ class OMNI2IR(GenericVisitor):
         if o.get('is_public') == 'true':
             _type = _type.clone(public=True)
 
-        if o.find('binding'):
+        if o.find('binding') is not None:
             bind_name = self.visit(o.find('binding/name'), **kwargs)
             bind_name_scope = scope.get_symbol_scope(bind_name.name)
 
@@ -852,6 +869,8 @@ class OMNI2IR(GenericVisitor):
             _type.public = True
         if o.get('is_save') == 'true':
             _type.save = True
+        if o.get('is_protected') == 'true':
+            _type.protected = True
         return _type
 
     def visit_FfunctionType(self, o, **kwargs):
@@ -1075,7 +1094,7 @@ class OMNI2IR(GenericVisitor):
     def visit_FifStatement(self, o, **kwargs):
         condition = self.visit(o.find('condition'), **kwargs)
         body = self.visit(o.find('then/body'), **kwargs)
-        if o.find('else'):
+        if o.find('else') is not None:
             else_body = self.visit(o.find('else/body'), **kwargs)
         else:
             else_body = ()
@@ -1303,8 +1322,9 @@ class OMNI2IR(GenericVisitor):
     def visit_FprintStatement(self, o, **kwargs):
         values = [self.visit(v, **kwargs) for v in o.find('valueList')]
         args = ', '.join(f'{v}' for v in values)
+        args = f", {args}" if values else ""
         fmt = o.attrib['format']
-        return ir.Intrinsic(text=f'print {fmt}, {args}', source=kwargs['source'])
+        return ir.Intrinsic(text=f'print {fmt}{args}', source=kwargs['source'])
 
     def visit_FformatDecl(self, o, **kwargs):
         fmt = f'FORMAT{o.attrib["format"]}'
