@@ -340,10 +340,11 @@ class Pattern:
 
         # We should now be left with no parentheses anymore and can build the new string
         # by using everything between these parenthesis "spans"
-        new_string = string[:spans[0][0]]
+        starts, ends = zip(*spans)
+        new_string = string[:min(starts)]
         for (_, start), (end, _) in zip(spans[:-1], spans[1:]):
             new_string += string[start+1:end]
-        new_string += string[spans[-1][1]+1:]
+        new_string += string[max(ends)+1:]
         return new_string
 
 
@@ -374,6 +375,9 @@ def parse_regex_source(source, parser_classes=None, scope=None):
     timeout_message = f'REGEX frontend timeout of {config["regex-frontend-timeout"]} s exceeded'
     with timeout(config['regex-frontend-timeout'], message=timeout_message):
         ir_ = Pattern.match_block_candidates(reader, candidates, parser_classes=parser_classes, scope=scope)
+
+    lines = (1, source.count('\n') + 1)
+    source = Source(lines, string=source)
     return ir.Section(body=as_tuple(ir_), source=source)
 
 
