@@ -14,9 +14,10 @@ import numpy as np
 import pytest
 
 from loki import (
-    Module, Subroutine, Sourcefile, BasicType, config, config_override
+    Function, Module, Subroutine, Sourcefile, BasicType, config,
+    config_override
 )
-from loki.build import jit_compile
+from loki.jit_build import jit_compile
 from loki.expression import symbols as sym
 from loki.frontend import available_frontends, OMNI, FP, HAVE_FP
 from loki.ir import nodes as ir, FindNodes, FindVariables
@@ -588,121 +589,6 @@ end subroutine test_inline_comments
     assert comments[0].text == comments[2].text == comments[3].text == ''
 
 
-@pytest.mark.parametrize('from_file', (True, False))
-@pytest.mark.parametrize('preprocess', (True, False))
-def test_source_sanitize_fp_source(tmp_path, from_file, preprocess):
-    """
-    Test that source sanitizing works as expected and postprocessing
-    rules are correctly applied
-    """
-    fcode = """
-subroutine some_routine(input_path)
-    implicit none
-    character(len=255), intent(in) :: input_path
-    integer :: ios, fu
-    write(*,*) "we print CPP value ", MY_VAR
-    ! In the following line the PP definition should be replace by '0'
-    ! or the actual line number
-    write(*,*) "We are in line ",__LINE__
-    open (action='read', file=TRIM(input_path), iostat=ios, newunit=fu)
-end subroutine some_routine
-""".strip()
-
-    if from_file:
-        filepath = tmp_path/'some_routine.F90'
-        filepath.write_text(fcode)
-        obj = Sourcefile.from_file(filepath, frontend=FP, preprocess=preprocess, defines=('MY_VAR=5',))
-    else:
-        obj = Sourcefile.from_source(fcode, frontend=FP, preprocess=preprocess, defines=('MY_VAR=5',))
-
-    if preprocess:
-        # CPP takes care of that
-        assert '"We are in line ", 8' in obj.to_fortran()
-        assert '"we print CPP value ", 5' in obj.to_fortran()
-    else:
-        # source sanitisation takes care of that
-        assert '"We are in line ", 0' in obj.to_fortran()
-        assert '"we print CPP value ", MY_VAR' in obj.to_fortran()
-
-    assert 'newunit=fu' in obj.to_fortran()
-
-
-@pytest.mark.parametrize('preprocess', (True, False))
-def test_source_sanitize_fp_subroutine(preprocess):
-    """
-    Test that source sanitizing works as expected and postprocessing
-    rules are correctly applied
-    """
-    fcode = """
-subroutine some_routine(input_path)
-    implicit none
-    character(len=255), intent(in) :: input_path
-    integer :: ios, fu
-    write(*,*) "we print CPP value ", MY_VAR
-    ! In the following line the PP definition should be replace by '0'
-    ! or the actual line number
-    write(*,*) "We are in line ",__LINE__
-    open (action='read', file=TRIM(input_path), iostat=ios, newunit=fu)
-end subroutine some_routine
-""".strip()
-
-    obj = Subroutine.from_source(fcode, frontend=FP, preprocess=preprocess, defines=('MY_VAR=5',))
-
-    if preprocess:
-        # CPP takes care of that
-        assert '"We are in line ", 8' in obj.to_fortran()
-        assert '"we print CPP value ", 5' in obj.to_fortran()
-    else:
-        # source sanitisation takes care of that
-        assert '"We are in line ", 0' in obj.to_fortran()
-        assert '"we print CPP value ", MY_VAR' in obj.to_fortran()
-
-    assert 'newunit=fu' in obj.to_fortran()
-
-
-@pytest.mark.parametrize('preprocess', (True, False))
-def test_source_sanitize_fp_module(preprocess):
-    """
-    Test that source sanitizing works as expected and postprocessing
-    rules are correctly applied
-    """
-    fcode = """
-module some_mod
-    implicit none
-    integer line = __LINE__ + MY_VAR
-contains
-subroutine some_routine(input_path)
-    implicit none
-    character(len=255), intent(in) :: input_path
-    integer :: ios, fu
-    write(*,*) "we print CPP value ", MY_VAR
-    ! In the following line the PP definition should be replace by '0'
-    ! or the actual line number
-    write(*,*) "We are in line ",__LINE__
-    open (action='read', file=TRIM(input_path), iostat=ios, newunit=fu)
-end subroutine some_routine
-end module some_mod
-""".strip()
-
-    obj = Module.from_source(fcode, frontend=FP, preprocess=preprocess, defines=('MY_VAR=5',))
-
-    if preprocess:
-        # CPP takes care of that
-        assert 'line = 3 + 5' in obj.to_fortran()
-        assert '"We are in line ", 12' in obj.to_fortran()
-        assert '"we print CPP value ", 5' in obj.to_fortran()
-    else:
-        # source sanitisation takes care of that
-        assert 'line = 0 + MY_VAR' in obj.to_fortran()
-        assert '"We are in line ", 0' in obj.to_fortran()
-        assert '"we print CPP value ", MY_VAR' in obj.to_fortran()
-
-    assert 'newunit=fu' in obj.to_fortran()
-
-
-# TODO: Add tests for source sanitizer with other frontends
-
-
 @pytest.mark.parametrize('frontend', available_frontends(xfail=[(OMNI, 'OMNI does not like Loki pragmas, yet!')]))
 def test_frontend_routine_variables_dimension_pragmas(frontend):
     """
@@ -872,6 +758,54 @@ end module mod_main
     assert var.type.module is mod_public
 
 
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_access_spec(tmp_path, frontend):
+    """
+    Check that access-spec statements are dealt with correctly.
+    """
+    code_mod_private_var_public = """
+module mod_private_var_public
+    private
+    integer :: var
+    public :: var
+end module mod_private_var_public
+    """
+    code_mod_public_var_private = """
+module mod_public_var_private
+    public
+    integer :: var
+    private :: var
+end module mod_public_var_private
+    """
+    code_mod_main = """
+module mod_main
+    use mod_private_var_public
+    use mod_public_var_private
+contains
+
+    subroutine test_routine()
+        integer :: result
+        result = var
+    end subroutine test_routine
+
+end module mod_main
+    """
+
+    mod_private_var_public = Module.from_source(code_mod_private_var_public, frontend=frontend, xmods=[tmp_path])
+    mod_public_var_private = Module.from_source(code_mod_public_var_private, frontend=frontend, xmods=[tmp_path])
+    mod_main = Module.from_source(
+        code_mod_main, frontend=frontend, definitions=[mod_private_var_public, mod_public_var_private], xmods=[tmp_path]
+    )
+    var = mod_main.subroutines[0].body.body[0].rhs
+    # Check if this is really our symbol
+    assert var.name == "var"
+    assert var.scope is mod_main
+    # Check if the symbol is imported
+    assert var.type.imported is True
+    # Check if the symbol comes from the mod_private_var_public module
+    assert var.type.module is mod_private_var_public
+
+
 @pytest.mark.parametrize('frontend', available_frontends(
     xfail=[(OMNI, 'OMNI does not like intrinsic shading for member functions!')]
 ))
@@ -953,25 +887,25 @@ end module test_intrinsics_mod
 def test_function_symbol_scoping(frontend):
     """ Check that the return symbol of a function has the right scope """
     fcode = """
-real function double_real(i)
+real(kind=8) function double_real(i)
   implicit none
   integer, intent(in) :: i
 
   double_real =  dble(i*2)
 end function double_real
 """
-    routine = Subroutine.from_source(fcode, frontend=frontend)
+    routine = Function.from_source(fcode, frontend=frontend)
 
-    rsym = routine.variable_map['double_real']
-    assert isinstance(rsym, sym.Scalar)
-    assert rsym.type.dtype == BasicType.REAL
-    assert rsym.scope == routine
+    rtyp = routine.symbol_attrs['double_real']
+    assert rtyp.dtype == BasicType.REAL
+    assert rtyp.kind == 8
 
     assigns = FindNodes(ir.Assignment).visit(routine.body)
     assert len(assigns) == 1
     assert assigns[0].lhs == 'double_real'
     assert isinstance(assigns[0].lhs, sym.Scalar)
     assert assigns[0].lhs.type.dtype == BasicType.REAL
+    assert assigns[0].lhs.type.kind == 8
     assert assigns[0].lhs.scope == routine
 
 
@@ -1054,3 +988,92 @@ end subroutine kernel
     assert c.shape[1].lower == 0
 
     assert all('*' in str(shape) for shape in [a.shape, b.shape, c.shape])
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_empty_print_statement(frontend):
+    """
+    Test if an empty print statement (PRINT *) is parsed correctly.
+    """
+    fcode = """
+SUBROUTINE test_routine()
+    IMPLICIT NONE
+    print *
+    ! Using single quotes to simplify the test comparison (see below)
+    print *, 'test_text'
+END SUBROUTINE test_routine
+    """.strip()
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    print_stmts = [
+        intr for intr in FindNodes(ir.Intrinsic).visit(routine.ir)
+        if 'print' in intr.text.lower()
+    ]
+    assert print_stmts[0].text.lower() == "print *"
+    # NOTE: OMNI always uses single quotes ('') to represent string data in PRINT statements
+    #       while fparser will mimic the quotes used in the parsed source code
+    assert print_stmts[1].text.lower() == "print *, 'test_text'"
+
+
+@pytest.mark.parametrize('frontend', available_frontends(xfail=[(OMNI, 'OMNI fails to read without full module')]))
+def test_select_type(frontend, tmp_path):
+    fcode = """
+module select_type_mod
+    use imported_type_mod, only: imported_type
+    implicit none
+    type, abstract :: base
+    end type base
+    type, extends(base) :: derived1
+        real :: val
+    end type derived1
+    type, extends(base) :: derived2
+        integer :: val
+    end type derived2
+contains
+    subroutine select_type_routine(arg, arg2)
+        class(base), intent(inout) :: arg
+        class(imported_type), intent(inout) :: arg2
+        select type( arg )
+            class is(derived1)
+                arg%val = 1.0
+            class is(derived2)
+                arg%val = 1
+            class default
+                print *, 'error'
+        end select
+        ! Some comment before the second select
+        select type( arg )
+            type is(base)
+                write(*,*) 'default'
+        end select
+        select type( arg2 )
+            ! inline comment
+            type is(imported_type)
+                print *, 'imported type'
+        end select
+    end subroutine select_type_routine
+end module select_type_mod
+    """.strip()
+
+    module = Module.from_source(fcode, frontend=frontend, xmods=[tmp_path])
+    tconds = FindNodes(ir.TypeConditional).visit(module['select_type_routine'].body)
+    assert len(tconds) == 3
+
+    assert tconds[0].expr == 'arg'
+    assert tconds[0].values == (
+        ('derived1', True), ('derived2', True)
+    )
+    assert len(tconds[0].bodies) == 2
+    assert len(tconds[0].else_body) == 1
+
+    assert tconds[1].expr == 'arg'
+    assert tconds[1].values == (('base', False),)
+    assert not tconds[1].else_body
+
+    assert tconds[2].expr == 'arg2'
+    assert tconds[2].values == (('imported_type', False),)
+    assert not tconds[2].else_body
+
+    comments = FindNodes(ir.Comment).visit(module['select_type_routine'].body)
+    assert len(comments) == 2
+    assert 'Some comment' in comments[0].text
+    assert 'inline comment' in comments[1].text
