@@ -43,7 +43,8 @@ __all__ = [
     'Allocation', 'Deallocation', 'Nullify',
     'Comment', 'CommentBlock', 'Pragma', 'PreprocessorDirective',
     'Import', 'VariableDeclaration', 'ProcedureDeclaration', 'DataDeclaration',
-    'StatementFunction', 'TypeDef', 'MultiConditional', 'Forall', 'MaskedStatement',
+    'StatementFunction', 'TypeDef', 'MultiConditional', 'TypeConditional',
+    'Forall', 'MaskedStatement',
     'Intrinsic', 'Enumeration', 'RawSource',
 ]
 
@@ -175,7 +176,7 @@ class Node:
         Pretty-print the node hierachy under this node.
         """
         # pylint: disable=import-outside-toplevel,cyclic-import
-        from loki.ir.pprint import pprint
+        from loki.backend.pprint import pprint
         pprint(self)
 
     def ir_graph(self, show_comments=False, show_expressions=False, linewidth=40, symgen=str):
@@ -1755,7 +1756,7 @@ class _MultiConditionalBase():
     """ Type definitions for :any:`MultiConditional` node type. """
 
     expr: Expression
-    values: Tuple[Any, ...]
+    values: Tuple[Tuple[Expression, ...], ...]
     bodies: Tuple[Any, ...]
     else_body: Tuple[Node, ...]
     name: Optional[str] = None
@@ -1764,7 +1765,7 @@ class _MultiConditionalBase():
 @dataclass_strict(frozen=True)
 class MultiConditional(LeafNode, _MultiConditionalBase):
     """
-    Internal representation of a multi-value conditional (eg. ``SELECT``).
+    Internal representation of a multi-value conditional (eg. ``SELECT CASE``).
 
     Parameters
     ----------
@@ -1784,6 +1785,16 @@ class MultiConditional(LeafNode, _MultiConditionalBase):
 
     _traversable = ['expr', 'values', 'bodies', 'else_body']
 
+    @field_validator('else_body', mode='before')
+    @classmethod
+    def ensure_tuple(cls, value):
+        return _sanitize_tuple(value)
+
+    @field_validator('values', 'bodies', mode='before')
+    @classmethod
+    def ensure_nested_tuple(cls, value):
+        return tuple(_sanitize_tuple(pair) for pair in as_tuple(value))
+
     def __post_init__(self):
         super().__post_init__()
         assert isinstance(self.expr, Expression)
@@ -1798,6 +1809,58 @@ class MultiConditional(LeafNode, _MultiConditionalBase):
         return f'MultiConditional::{label} {str(self.expr)}'
 
 
+@dataclass_strict(frozen=True)
+class _TypeConditionalBase():
+    """ Type definitions for :any:`TypeConditional` node type. """
+
+    expr: Expression
+    values: Tuple[Tuple[Expression, bool], ...]
+    bodies: Tuple[Any, ...]
+    else_body: Tuple[Node, ...]
+    name: Optional[str] = None
+
+
+@dataclass_strict(frozen=True)
+class TypeConditional(LeafNode, _TypeConditionalBase):
+    """
+    Internal representation of a multi-type conditional (eg. ``SELECT TYPE``).
+
+    Parameters
+    ----------
+    expr : :any:`pymbolic.primitives.Expression`
+        The expression that is evaluated to choose the appropriate type case.
+    values : tuple of 2-tuple with (:any:`pymbolic.primitives.Expression`, bool)
+        The list of values, a tuple for each case consisting of the type name and
+        a bool value to indicate if this is a derived/polymorphic type case (in Fortran,
+        this yields the difference between ``TYPE IS`` and ``CLASS IS``)
+    bodies : tuple of tuple
+        The corresponding bodies for each case.
+    else_body : tuple
+        The body for the ``DEFAULT`` case.
+    name : str, optional
+        The construct-name of the multi conditional in the original source.
+    **kwargs : optional
+        Other parameters that are passed on to the parent class constructor.
+    """
+
+    _traversable = ['expr', 'values', 'bodies', 'else_body']
+
+    def __post_init__(self):
+        super().__post_init__()
+        assert isinstance(self.expr, Expression)
+        assert is_iterable(self.values)
+        assert all(
+            isinstance(v, tuple) and len(v) == 2 and isinstance(v[0], Expression) and isinstance(v[1], bool)
+            for v in self.values
+        )
+        assert is_iterable(self.bodies) and all(is_iterable(b) for b in self.bodies)
+        assert is_iterable(self.else_body)
+
+    def __repr__(self):
+        label = f' {self.name}' if self.name else ''
+        return f'TypeConditional::{label} {str(self.expr)}'
+
+
 @dataclass_strict(frozen=True)
 class _ForallBase():
     """ Type definition for :any:`Forall` node type. """
