@@ -9,7 +9,7 @@
 Contains the declaration of :any:`Module` to represent Fortran modules.
 """
 from loki.frontend import (
-    get_fparser_node, parse_omni_ast, parse_ofp_ast, parse_fparser_ast,
+    get_fparser_node, parse_omni_ast, parse_fparser_ast,
     parse_regex_source
 )
 from loki.ir import (
@@ -143,31 +143,6 @@ class Module(ProgramUnit):
             type_map=type_map, scope=parent
         )
 
-    @classmethod
-    def from_ofp(cls, ast, raw_source, definitions=None, pp_info=None, parent=None):
-        """
-        Create :any:`Module` from :any:`OFP` parse tree
-
-        Parameters
-        ----------
-        ast :
-            The OFP parse tree
-        raw_source : str
-            Fortran source string
-        definitions : list
-            List of external :any:`Module` to provide derived-type and procedure declarations
-        pp_info :
-            Preprocessing info as obtained by :any:`sanitize_input`
-        parent : :any:`Scope`, optional
-            The enclosing parent scope of the module.
-        """
-        if ast.tag != 'module':
-            ast = ast.find('file/module')
-        return parse_ofp_ast(
-            ast=ast, pp_info=pp_info, raw_source=raw_source,
-            definitions=definitions, scope=parent
-        )
-
     @classmethod
     def from_fparser(cls, ast, raw_source, definitions=None, pp_info=None, parent=None):
         """
