@@ -263,6 +263,7 @@ class Sourcefile:
 
         lines = (1, raw_source.count('\n') + 1)
         source = Source(lines, string=raw_source, file=path)
+        ir._update(source=source)
         return cls(path=path, ir=ir, ast=ast, source=source)
 
     @classmethod
@@ -426,7 +427,7 @@ class Sourcefile:
 
     def to_fortran(self, conservative=False, cuf=False, style=None):
         if cuf:
-            return cufgen(self, conservative=conservative, style=style)
+            return cufgen(self, style=style)
         return fgen(self, conservative=conservative, style=style)
 
     @property
