@@ -19,11 +19,9 @@ from loki.transformations.data_offload import * # noqa
 from loki.transformations.drhook import * # noqa
 from loki.transformations.extract import * # noqa
 from loki.transformations.field_api import * # noqa
-from loki.transformations.hoist_variables import * # noqa
 from loki.transformations.idempotence import * # noqa
 from loki.transformations.inline import * # noqa
 from loki.transformations.parametrise import * # noqa
-from loki.transformations.raw_stack_allocator import * # noqa
 from loki.transformations.remove_code import * # noqa
 from loki.transformations.sanitise import * # noqa
 from loki.transformations.single_column import * # noqa
@@ -31,7 +29,6 @@ from loki.transformations.transpile import * # noqa
 from loki.transformations.transform_derived_types import * # noqa
 from loki.transformations.transform_loop import * # noqa
 from loki.transformations.transform_region import * # noqa
-from loki.transformations.pool_allocator import * # noqa
 from loki.transformations.utilities import * # noqa
 from loki.transformations.block_index_transformations import * # noqa
 from loki.transformations.split_read_write import * # noqa
@@ -40,3 +37,4 @@ from loki.transformations.routine_signatures import * # noqa
 from loki.transformations.parallel import * # noqa
 from loki.transformations.dependency import * # noqa
 from loki.transformations.pragma_model import * # noqa
+from loki.transformations.temporaries import * # noqa
