@@ -115,6 +115,8 @@ class DependencyTransformation(Transformation):
         """
         role = kwargs.get('role')
 
+        # remember/keep track of the module subroutines (even if some of those are removed)
+        routines = tuple(routine.name.lower() for routine in module.subroutines)
         if role == 'kernel':
             # Change the name of kernel modules
             module.name = self.derive_module_name(module.name)
@@ -133,6 +135,20 @@ class DependencyTransformation(Transformation):
         if self.replace_ignore_items and (item := kwargs.get('item')):
             targets += tuple(str(i).lower() for i in item.ignore)
         self.rename_imports(module, imports=module.imports, targets=targets)
+        active_nodes = None
+        if self.remove_inactive_items and not kwargs.get('items') is None:
+            active_nodes = [item.scope_ir.name.lower() for item in kwargs['items']]
+        # rename target names in an access spec for both public and private access specs 
+        if module.public_access_spec:
+            module.public_access_spec = self.rename_access_spec_names(
+                module.public_access_spec, targets=targets, active_nodes=active_nodes,
+                routines=routines
+            )
+        if module.private_access_spec:
+            module.private_access_spec = self.rename_access_spec_names(
+                module.private_access_spec, targets=targets, active_nodes=active_nodes,
+                routines=routines
+            )
 
     def transform_subroutine(self, routine, **kwargs):
         """
@@ -329,6 +345,34 @@ class DependencyTransformation(Transformation):
         if import_map:
             source.spec = Transformer(import_map).visit(source.spec)
 
+    def rename_access_spec_names(self, access_spec, targets=None, active_nodes=None, routines=None):
+        """
+        Rename target names in an access spec
+        
+        For all names in the access spec that are contained in :data:`targets`, rename them as 
+        ``{name}{self.suffix}``. If :data:`active_nodes` are given, then all names
+        that are not in the list of active nodes, are being removed from the list.
+        Parameters
+        ----------
+        access_spec : list of str
+            List of names from an access spec
+        targets : list of str
+            Optional list of subroutine names for which to modify access specs
+        active_nodes : list of str
+            Optional list of active nodes
+        routines : list of :any:`Subroutine`
+            Optional list of subroutines
+        """
+        module_routines = routines or ()
+        if active_nodes is not None:
+            access_spec = tuple(elem for elem in access_spec if elem in active_nodes
+                                or elem.lower() not in module_routines)
+        return tuple(
+            f'{elem}{self.suffix}' if elem.lower() in module_routines and (not targets or elem in targets)
+            else elem
+            for elem in access_spec
+        )
+
     def rename_interfaces(self, intfs, targets=None):
         """
         Update explicit interfaces to actively transformed subroutines.
