@@ -12,9 +12,9 @@ from loki.analyse import dataflow_analysis_attached
 from loki.batch import Transformation, ProcedureItem, ModuleItem
 from loki.expression import Scalar, Array
 from loki.ir import (
-    FindNodes, CallStatement, Pragma, Import, Comment,
-    Transformer, get_pragma_parameters,
-    FindInlineCalls, SubstituteExpressions
+FindNodes, CallStatement, Pragma, Import, Comment,
+Transformer, get_pragma_parameters,
+FindInlineCalls, SubstituteExpressions
 )
 from loki.logging import warning
 from loki.tools import as_tuple, flatten, CaseInsensitiveDict, CaseInsensitiveDefaultDict
@@ -22,368 +22,371 @@ from loki.types import DerivedType
 
 
 __all__ = [
-    'GlobalVariableAnalysis', 'GlobalVarOffloadTransformation',
-    'GlobalVarHoistTransformation',
+'GlobalVariableAnalysis', 'GlobalVarOffloadTransformation',
+'GlobalVarHoistTransformation',
 ]
 
 
 class GlobalVariableAnalysis(Transformation):
-    """
-    Transformation pass to analyse the declaration and use of (global) module variables.
+"""
+Transformation pass to analyse the declaration and use of (global) module variables.
 
-    This analysis is a requirement before applying :any:`GlobalVarOffloadTransformation`.
+This analysis is a requirement before applying :any:`GlobalVarOffloadTransformation`.
 
-    Collect data in :any:`Item.trafo_data` for :any:`ProcedureItem` and
-    :any:`ModuleItem` items and store analysis results under the
-    provided :data:`key` (default: ``'GlobalVariableAnalysis'``) in the
-    items' ``trafo_data``.
+Collect data in :any:`Item.trafo_data` for :any:`ProcedureItem` and
+:any:`ModuleItem` items and store analysis results under the
+provided :data:`key` (default: ``'GlobalVariableAnalysis'``) in the
+items' ``trafo_data``.
 
-    For procedures, use the the Loki dataflow analysis functionality to compile
-    a list of used and/or defined variables (i.e., read and/or written).
-    Store these under the keys ``'uses_symbols'`` and ``'defines_symbols'``,
-    respectively.
+For procedures, use the the Loki dataflow analysis functionality to compile
+a list of used and/or defined variables (i.e., read and/or written).
+Store these under the keys ``'uses_symbols'`` and ``'defines_symbols'``,
+respectively.
 
-    For modules/:any:`ModuleItem`, store the list of variables declared in the
-    module under the key ``'declares'`` and out of this the subset of variables that
-    need offloading to device under the key ``'offload'``.
+For modules/:any:`ModuleItem`, store the list of variables declared in the
+module under the key ``'declares'`` and out of this the subset of variables that
+need offloading to device under the key ``'offload'``.
 
-    Note that in every case, the full variable symbols are stored to allow access to
-    type information in transformations using the analysis data.
+Note that in every case, the full variable symbols are stored to allow access to
+type information in transformations using the analysis data.
 
-    The generated trafo_data has the following schema::
+The generated trafo_data has the following schema::
 
-        ModuleItem: {
-            'declares': set(Variable, Variable, ...),
-            'offload': set(Variable, ...)
-        }
+ModuleItem: {
+    'declares': set(Variable, Variable, ...),
+    'offload': set(Variable, ...)
+}
 
-        ProcedureItem: {
-            'uses_symbols': set( (Variable, '<module_name>'), (Variable, '<module_name>'), ...),
-            'defines_symbols': set((Variable, '<module_name>'), (Variable, '<module_name>'), ...)
-        }
+ProcedureItem: {
+    'uses_symbols': set( (Variable, '<module_name>'), (Variable, '<module_name>'), ...),
+    'defines_symbols': set((Variable, '<module_name>'), (Variable, '<module_name>'), ...)
+}
 
-    Parameters
-    ----------
-    key : str, optional
-        Specify a different identifier under which trafo_data is stored
-    """
+Parameters
+----------
+key : str, optional
+Specify a different identifier under which trafo_data is stored
+"""
 
-    _key = 'GlobalVariableAnalysis'
-    """Default identifier for trafo_data entry"""
+_key = 'GlobalVariableAnalysis'
+"""Default identifier for trafo_data entry"""
 
-    reverse_traversal = True
-    """Traversal from the leaves upwards, i.e., modules with global variables are processed first,
-    then kernels using them before the driver."""
+reverse_traversal = True
+"""Traversal from the leaves upwards, i.e., modules with global variables are processed first,
+then kernels using them before the driver."""
 
-    item_filter = (ProcedureItem, ModuleItem)
-    """Process procedures and modules with global variable declarations."""
+item_filter = (ProcedureItem, ModuleItem)
+"""Process procedures and modules with global variable declarations."""
 
-    def __init__(self, key=None):
-        if key:
-            self._key = key
+def __init__(self, key=None):
+if key:
+    self._key = key
 
-    def transform_module(self, module, **kwargs):
-        if 'item' not in kwargs:
-            raise RuntimeError('Cannot apply GlobalVariableAnalysis without item to store analysis data')
+def transform_module(self, module, **kwargs):
+if 'item' not in kwargs:
+    raise RuntimeError('Cannot apply GlobalVariableAnalysis without item to store analysis data')
 
-        item = kwargs['item']
+item = kwargs['item']
 
-        # Gather all module variables and filter out parameters
-        variables = {var for var in module.variables if not var.type.parameter}
+# Gather all module variables and filter out parameters
+variables = {var for var in module.variables if not var.type.parameter}
 
-        # Initialize and store trafo data
-        item.trafo_data[self._key] = {
-            'declares': variables,
-            'offload': set()
-        }
+# Initialize and store trafo data
+item.trafo_data[self._key] = {
+    'declares': variables,
+    'offload': set()
+}
 
-    def transform_subroutine(self, routine, **kwargs):
-        if 'item' not in kwargs:
-            raise RuntimeError('Cannot apply GlobalVariableAnalysis without item to store analysis data')
-        if 'sub_sgraph' not in kwargs:
-            raise RuntimeError(('Cannot apply GlobalVariableAnalysis without information'
-                ' about successors to store offload analysis data'))
+def transform_subroutine(self, routine, **kwargs):
+if 'item' not in kwargs:
+    raise RuntimeError('Cannot apply GlobalVariableAnalysis without item to store analysis data')
+if 'sub_sgraph' not in kwargs:
+    raise RuntimeError(('Cannot apply GlobalVariableAnalysis without information'
+	' about successors to store offload analysis data'))
 
-        item = kwargs['item']
-        sub_sgraph = kwargs.get('sub_sgraph', None)
-        successors = sub_sgraph.successors(item) if sub_sgraph is not None else ()
+item = kwargs['item']
+sub_sgraph = kwargs.get('sub_sgraph', None)
+successors = sub_sgraph.successors(item) if sub_sgraph is not None else ()
 
-        # Gather all symbols imported in this routine or parent scopes
-        import_map = CaseInsensitiveDict()
-        scope = routine
-        while scope:
-            import_map.update(scope.import_map)
-            scope = scope.parent
+# Gather all symbols imported in this routine or parent scopes
+import_map = CaseInsensitiveDict()
+scope = routine
+while scope:
+    import_map.update(scope.import_map)
+    scope = scope.parent
 
-        with dataflow_analysis_attached(routine):
-            # Gather read and written symbols that have been imported
-            uses_imported_symbols = {
-                var for var in routine.body.uses_symbols
-                if var.name in import_map or (var.parent and var.parents[0].name in import_map)
-            }
-            uses_imported_symbols |= {
-                var for var in routine.spec.uses_symbols
-                if var.name in import_map or (var.parent and var.parents[0].name in import_map)
-            }
-            defines_imported_symbols = {
-                var for var in routine.body.defines_symbols
-                if var.name in import_map or (var.parent and var.parents[0].name in import_map)
-            }
+with dataflow_analysis_attached(routine):
+    # Gather read and written symbols that have been imported
+    uses_imported_symbols = {
+	var for var in routine.body.uses_symbols
+	if var.name in import_map or (var.parent and var.parents[0].name in import_map)
+    }
+    uses_imported_symbols |= {
+	var for var in routine.spec.uses_symbols
+	if var.name in import_map or (var.parent and var.parents[0].name in import_map)
+    }
+    defines_imported_symbols = {
+	var for var in routine.body.defines_symbols
+	if var.name in import_map or (var.parent and var.parents[0].name in import_map)
+    }
 
-            # Filter out type and procedure imports by restricting to Scalar and Array symbols
-            uses_imported_symbols = {var for var in uses_imported_symbols if isinstance(var, (Scalar, Array))}
-            defines_imported_symbols = {var for var in defines_imported_symbols if isinstance(var, (Scalar, Array))}
+    # Filter out type and procedure imports by restricting to Scalar and Array symbols
+    uses_imported_symbols = {var for var in uses_imported_symbols if isinstance(var, (Scalar, Array))}
+    defines_imported_symbols = {var for var in defines_imported_symbols if isinstance(var, (Scalar, Array))}
 
-            def _map_var_to_module(var):
-                if var.parent:
-                    module = var.parents[0].type.module
-                    module_var = module.variable_map[var.parents[0].name]
-                    dimensions = getattr(module_var, 'dimensions', None)
-                    for child in chain(var.parents[1:], (var,)):
-                        module_var = child.clone(
-                            name=f'{module_var.name}%{child.name}',
-                            parent=module_var,
-                            scope=module_var.scope
-                        )
-                    return (module_var.clone(dimensions=dimensions), module.name.lower())
-                module = var.type.module
-                return (module.variable_map[var.name], module.name.lower())
+    def _map_var_to_module(var):
+	if var.parent:
+	    module = var.parents[0].type.module
+	    module_var = module.variable_map[var.parents[0].name]
+	    dimensions = getattr(module_var, 'dimensions', None)
+	    for child in chain(var.parents[1:], (var,)):
+		module_var = child.clone(
+		    name=f'{module_var.name}%{child.name}',
+		    parent=module_var,
+		    scope=module_var.scope
+		)
+	    return (module_var.clone(dimensions=dimensions), module.name.lower())
+	module = var.type.module
+	return (module.variable_map[var.name], module.name.lower())
 
-            # Store symbol lists in trafo data
-            item.trafo_data[self._key] = {}
-            item.trafo_data[self._key]['uses_symbols'] = {
-                _map_var_to_module(var) for var in uses_imported_symbols
-            }
-            item.trafo_data[self._key]['defines_symbols'] = {
-                _map_var_to_module(var) for var in defines_imported_symbols
-            }
+    # Store symbol lists in trafo data
+    item.trafo_data[self._key] = {}
+    item.trafo_data[self._key]['uses_symbols'] = {
+	_map_var_to_module(var) for var in uses_imported_symbols
+    }
+    item.trafo_data[self._key]['defines_symbols'] = {
+	_map_var_to_module(var) for var in defines_imported_symbols
+    }
 
-        # Amend analysis data with data from successors
-        # Note: This is a temporary workaround for the incomplete list of successor items
-        # provided by the current scheduler implementation
-        for successor in successors:
-            if isinstance(successor, ProcedureItem):
-                item.trafo_data[self._key]['uses_symbols'] |= successor.trafo_data[self._key]['uses_symbols']
-                item.trafo_data[self._key]['defines_symbols'] |= successor.trafo_data[self._key]['defines_symbols']
+# Amend analysis data with data from successors
+# Note: This is a temporary workaround for the incomplete list of successor items
+# provided by the current scheduler implementation
+for successor in successors:
+    if isinstance(successor, ProcedureItem):
+	item.trafo_data[self._key]['uses_symbols'] |= successor.trafo_data[self._key]['uses_symbols']
+	item.trafo_data[self._key]['defines_symbols'] |= successor.trafo_data[self._key]['defines_symbols']
 
 
 class GlobalVarOffloadTransformation(Transformation):
-    """
-    Transformation to insert offload directives for module variables used in device routines
+"""
+Transformation to insert offload directives for module variables used in device routines
 
-    Currently, only OpenACC data offloading is supported.
+Currently, only OpenACC data offloading is supported.
 
-    This requires a prior analysis pass with :any:`GlobalVariableAnalysis` to collect
-    the relevant global variable use information.
+This requires a prior analysis pass with :any:`GlobalVariableAnalysis` to collect
+the relevant global variable use information.
 
-    The offload directives are inserted by replacing ``!$loki update_device`` and
-    ``!$loki update_host`` pragmas in the driver's source code. Importantly, no offload
-    directives are added if these pragmas have not been added to the original source code!
+The offload directives are inserted by replacing ``!$loki update_device`` and
+``!$loki update_host`` pragmas in the driver's source code. Importantly, no offload
+directives are added if these pragmas have not been added to the original source code!
 
-    For global variables, the device-side declarations are added in :meth:`transform_module`.
-    For driver procedures, the data offload and pull-back directives are added in
-    the utility method :meth:`process_driver`, which is invoked by :meth:`transform_subroutine`.
+For global variables, the device-side declarations are added in :meth:`transform_module`.
+For driver procedures, the data offload and pull-back directives are added in
+the utility method :meth:`process_driver`, which is invoked by :meth:`transform_subroutine`.
 
-    For example, the following code:
+For example, the following code:
 
-    .. code-block:: fortran
+.. code-block:: fortran
 
-        module moduleB
-           real :: var2
-           real :: var3
-        end module moduleB
+module moduleB
+   real :: var2
+   real :: var3
+end module moduleB
 
-        module moduleC
-           real :: var4
-           real :: var5
-        end module moduleC
+module moduleC
+   real :: var4
+   real :: var5
+end module moduleC
 
-        subroutine driver()
-        implicit none
+subroutine driver()
+implicit none
 
-        !$loki update_device
-        !$acc serial
-        call kernel()
-        !$acc end serial
-        !$loki update_host
+!$loki update_device
+!$acc serial
+call kernel()
+!$acc end serial
+!$loki update_host
 
-        end subroutine driver
+end subroutine driver
 
-        subroutine kernel()
-        use moduleB, only: var2,var3
-        use moduleC, only: var4,var5
-        implicit none
-        !$acc routine seq
+subroutine kernel()
+use moduleB, only: var2,var3
+use moduleC, only: var4,var5
+implicit none
+!$acc routine seq
 
-        var4 = var2
-        var5 = var3
+var4 = var2
+var5 = var3
 
-        end subroutine kernel
+end subroutine kernel
 
-    is transformed to:
+is transformed to:
 
-    .. code-block:: fortran
+.. code-block:: fortran
 
-        module moduleB
-           real :: var2
-           real :: var3
-          !$acc declare create(var2)
-          !$acc declare create(var3)
-        end module moduleB
+module moduleB
+   real :: var2
+   real :: var3
+  !$acc declare create(var2)
+  !$acc declare create(var3)
+end module moduleB
 
-        module moduleC
-           real :: var4
-           real :: var5
-          !$acc declare create(var4)
-          !$acc declare create(var5)
-        end module moduleC
+module moduleC
+   real :: var4
+   real :: var5
+  !$acc declare create(var4)
+  !$acc declare create(var5)
+end module moduleC
 
-        subroutine driver()
-        implicit none
+subroutine driver()
+implicit none
 
-        !$acc update device( var2,var3 )
-        !$acc serial
-        call kernel()
-        !$acc end serial
-        !$acc update self( var4,var5 )
+!$acc update device( var2,var3 )
+!$acc serial
+call kernel()
+!$acc end serial
+!$acc update self( var4,var5 )
 
-        end subroutine driver
+end subroutine driver
 
-    Nested Fortran derived-types and arrays of derived-types are not currently supported.
-    If such an import is encountered, only the device-side declaration will be added to the
-    relevant module file, and the offload instructions will have to manually be added afterwards.
-    """
+Nested Fortran derived-types and arrays of derived-types are not currently supported.
+If such an import is encountered, only the device-side declaration will be added to the
+relevant module file, and the offload instructions will have to manually be added afterwards.
+"""
 
-    # Include module variable imports in the underlying graph
-    # connectivity for traversal with the Scheduler
-    item_filter = (ProcedureItem, ModuleItem)
+# Include module variable imports in the underlying graph
+# connectivity for traversal with the Scheduler
+item_filter = (ProcedureItem, ModuleItem)
 
-    def __init__(self, key=None):
-        self._key = key or GlobalVariableAnalysis._key
+def __init__(self, key=None):
+self._key = key or GlobalVariableAnalysis._key
 
-    def transform_module(self, module, **kwargs):
-        """
-        Add device-side declarations for imported variables
-        """
-        if 'item' not in kwargs:
-            raise RuntimeError('Cannot apply GlobalVarOffloadTransformation without trafo_data in item')
+def transform_module(self, module, **kwargs):
+"""
+Add device-side declarations for imported variables
+"""
+if 'item' not in kwargs:
+    raise RuntimeError('Cannot apply GlobalVarOffloadTransformation without trafo_data in item')
 
-        item = kwargs['item']
+item = kwargs['item']
 
-        # Check for already declared offloads
-        acc_pragmas = [pragma for pragma in FindNodes(Pragma).visit(module.spec) if pragma.keyword.lower() == 'acc']
-        acc_pragma_parameters = get_pragma_parameters(acc_pragmas, starts_with='declare', only_loki_pragmas=False)
-        declared_variables = set(flatten([
-            v.replace(' ','').lower().split()
-            for v in as_tuple(acc_pragma_parameters.get('create'))
-        ]))
+# Check for already declared offloads
+acc_pragmas = [pragma for pragma in FindNodes(Pragma).visit(module.spec) if pragma.keyword.lower() == 'acc']
+acc_pragma_parameters = get_pragma_parameters(acc_pragmas, starts_with='declare', only_loki_pragmas=False)
+declared_variables = set(flatten([
+    v.replace(' ','').lower().split()
+    for v in as_tuple(acc_pragma_parameters.get('create'))
+]))
 
-        # Build list of symbols to be offloaded (discard variables being parameter)
-        offload_variables = {
-            var.parents[0] if var.parent else var
-            for var in item.trafo_data[self._key].get('offload', ()) if not var.type.parameter
-        }
+# Build list of symbols to be offloaded (discard variables being parameter)
+offload_variables = {
+    var.parents[0] if var.parent else var
+    for var in item.trafo_data[self._key].get('offload', ()) if not var.type.parameter
+}
 
-        if (invalid_vars := offload_variables - set(module.variables)):
-            raise RuntimeError(f'Invalid variables in offload analysis: {", ".join(v.name for v in invalid_vars)}')
+if (invalid_vars := offload_variables - set(module.variables)):
+    raise RuntimeError(f'Invalid variables in offload analysis: {", ".join(v.name for v in invalid_vars)}')
 
-        # Add ACC declare pragma for offload variables that are not yet declared
-        offload_variables = offload_variables - declared_variables
-        if offload_variables:
-            module.spec.append(
-                Pragma(keyword='loki', content=f'create device({", ".join(v.name for v in offload_variables)})')
-            )
+# Add ACC declare pragma for offload variables that are not yet declared
+offload_variables = offload_variables - declared_variables
+if offload_variables:
+    module.spec.append(
+	Pragma(keyword='loki', content=f'create device({", ".join(v.name for v in offload_variables)})')
+    )
 
-    def transform_subroutine(self, routine, **kwargs):
-        """
-        Add data offload and pull-back directives to the driver
-        """
-        role = kwargs.get('role')
-        item = kwargs['item']
-        sub_sgraph = kwargs.get('sub_sgraph', None)
-        successors = sub_sgraph.successors(item) if sub_sgraph is not None else ()
+def transform_subroutine(self, routine, **kwargs):
+"""
+Add data offload and pull-back directives to the driver
+"""
+role = kwargs.get('role')
+item = kwargs['item']
+sub_sgraph = kwargs.get('sub_sgraph', None)
+successors = sub_sgraph.successors(item) if sub_sgraph is not None else ()
 
-        if role == 'driver':
-            self.process_driver(routine, successors)
-        elif role == 'kernel':
-            self.process_kernel(item, successors)
+if role == 'driver':
+    self.process_driver(routine, successors)
+elif role == 'kernel':
+    self.process_kernel(item, successors)
 
-    def process_kernel(self, item, successors):
-        """
-        Propagate offload requirement to the items of the global variables
-        """
-        successors_map = CaseInsensitiveDict(
-            (item.name, item) for item in successors if isinstance(item, ModuleItem)
-        )
-        for var, module in chain(
-            item.trafo_data[self._key]['uses_symbols'],
-            item.trafo_data[self._key]['defines_symbols']
-        ):
-            if var.type.parameter:
-                continue
-            if successor := successors_map.get(module):
-                successor.trafo_data[self._key]['offload'].add(var)
+def process_kernel(self, item, successors):
+"""
+Propagate offload requirement to the items of the global variables
+"""
+successors_map = CaseInsensitiveDict(
+    (item.name, item) for item in successors if isinstance(item, ModuleItem)
+)
+for var, module in chain(
+    item.trafo_data[self._key]['uses_symbols'],
+    item.trafo_data[self._key]['defines_symbols']
+):
+    if var.type.parameter:
+	continue
+    if successor := successors_map.get(module):
+	successor.trafo_data[self._key]['offload'].add(var)
 
-    def process_driver(self, routine, successors):
-        """
-        Add data offload and pullback directives
+def process_driver(self, routine, successors):
+"""
+Add data offload and pullback directives
 
-        List of variables that requires offloading is obtained from the analysis data
-        stored for each successor in :data:`successors`.
-        """
-        # Empty lists for update directives
-        update_device = ()
-        update_host = ()
+List of variables that requires offloading is obtained from the analysis data
+stored for each successor in :data:`successors`.
+"""
+# Empty lists for update directives
+update_device = ()
+update_host = ()
 
-        # Combine analysis data across successor items
-        defines_symbols = set()
-        uses_symbols = set()
-        for item in successors:
-            defines_symbols |= item.trafo_data.get(self._key, {}).get('defines_symbols', set())
-            uses_symbols |= item.trafo_data.get(self._key, {}).get('uses_symbols', set())
-            # discard variables being parameter
-            parameters = {(var, module) for var, module in uses_symbols if var.type.parameter}
-            uses_symbols ^= parameters
+# Combine analysis data across successor items
+defines_symbols = set()
+uses_symbols = set()
+for item in successors:
+    defines_symbols |= item.trafo_data.get(self._key, {}).get('defines_symbols', set())
+    uses_symbols |= item.trafo_data.get(self._key, {}).get('uses_symbols', set())
+    # discard variables being parameter
+    parameters = {(var, module) for var, module in uses_symbols if var.type.parameter}
+    uses_symbols ^= parameters
 
-        # Filter out arrays of derived types and nested derived types
-        # For these, automatic offloading is currently not supported
-        exclude_symbols = set()
-        for var_, module in chain(defines_symbols, uses_symbols):
-            var = var_.parents[0] if var_.parent else var_
-            if not isinstance(var.type.dtype, DerivedType):
-                continue
-            if isinstance(var, Array):
-                exclude_symbols.add(var)
-                warning((
-                    '[Loki::GlobalVarOffloadTransformation] '
-                    f'Automatic offloading of derived type arrays not implemented: {var} in {routine.name}'
-                ))
-            if any(isinstance(v.type.dtype, DerivedType) for v in var.type.dtype.typedef.variables):
-                exclude_symbols.add(var)
-                warning((
-                    '[Loki::GlobalVarOffloadTransformation] '
-                    f'Automatic offloading of nested derived types not implemented: {var} in {routine.name}'
-                ))
+# Filter out arrays of derived types and nested derived types
+# For these, automatic offloading is currently not supported
+exclude_symbols = set()
+for var_, module in chain(defines_symbols, uses_symbols):
+    var = var_.parents[0] if var_.parent else var_
+    if not isinstance(var.type.dtype, DerivedType):
+	continue
+    if isinstance(var, Array):
+	exclude_symbols.add(var)
+	warning((
+	    '[Loki::GlobalVarOffloadTransformation] '
+	    f'Automatic offloading of derived type arrays not implemented: {var} in {routine.name}'
+	))
+    if any(isinstance(v.type.dtype, DerivedType) for v in var.type.dtype.typedef.variables):
+	exclude_symbols.add(var)
+	warning((
+	    '[Loki::GlobalVarOffloadTransformation] '
+	    f'Automatic offloading of nested derived types not implemented: {var} in {routine.name}'
+	))
 
-        uses_symbols = {
-            (var, module) for var, module in uses_symbols
-            if var not in exclude_symbols and not (var.parent and var.parents[0] in exclude_symbols)
-        }
-        defines_symbols = {
-            (var, module) for var, module in defines_symbols
-            if var not in exclude_symbols and not (var.parent and var.parents[0] in exclude_symbols)
-        }
+uses_symbols = {
+    (var, module) for var, module in uses_symbols
+    if var not in exclude_symbols and not (var.parent and var.parents[0] in exclude_symbols)
+}
+defines_symbols = {
+    (var, module) for var, module in defines_symbols
+    if var not in exclude_symbols and not (var.parent and var.parents[0] in exclude_symbols)
+}
 
-        # All variables that are used in a kernel need a host-to-device transfer
-        if uses_symbols:
-            update_variables = {
-                v for v, _ in uses_symbols
-                if not (v.parent or isinstance(v.type.dtype, DerivedType))
-            }
-            copyin_variables = {v for v, _ in uses_symbols if v.parent}
-            if update_variables:
-                update_device += (
-                    Pragma(keyword='loki', content=f'update device({", ".join(v.name for v in update_variables)})'),
+# All variables that are used in a kernel need a host-to-device transfer
+if uses_symbols:
+    update_variables = {
+	v for v, _ in uses_symbols
+	if not (v.parent or isinstance(v.type.dtype, DerivedType))
+    }
+    copyin_variables = {v for v, _ in uses_symbols if v.parent}
+    if update_variables:
+	update_device += (
+	    Pragma(keyword='loki', content=f'update device({", ".join(v.name for v in update_variables)})'),
+                    # this shouldn't be necessary but is currently necessary because of a bug in OpenMP
+                    Pragma(keyword='loki',
+                        content=f'omp-update-global-vars in({", ".join(v.name for v in update_variables)})'),
                 )
             if copyin_variables:
                 content = f'unstructured-data in({", ".join(v.name for v in copyin_variables)})'
