@@ -188,6 +188,7 @@ subroutine driver(dims, struct, array_arg, geometry, variable)
    use kernel_mod, only : kernel
    use nested_kernel_write_mod, only: nested_kernel_write
    use type_def_mod, only: struct_type, dims_type, geom_type, other_variable_type
+   use iso_fortran_env, only : real64
    implicit none
 
    type(dims_type), intent(in) :: dims
@@ -196,7 +197,7 @@ subroutine driver(dims, struct, array_arg, geometry, variable)
    type(geom_type), intent(in) :: geometry
    type(other_variable_type), intent(inout) :: variable
    type(dims_type) :: local_dims
-   integer :: ibl
+   integer :: ibl, ij
 
    local_dims = dims
 
@@ -204,6 +205,9 @@ subroutine driver(dims, struct, array_arg, geometry, variable)
 !$loki data private(local_dims) present(geometry) write(struct%c%p)
    do ibl=1,local_dims%ngpblks
      local_dims%kbl = ibl
+     ij = 0
+
+     variable%vt0_field(:,:,ibl) = 0._real64
 
      call kernel(geometry, local_dims, struct, variable)
      call nested_kernel_write(struct%e%p(:,:,local_dims%kbl))
@@ -214,6 +218,9 @@ subroutine driver(dims, struct, array_arg, geometry, variable)
 !$loki data private(local_dims) write(struct%c%p)
    do ibl=1,local_dims%ngpblks
      local_dims%kbl = ibl
+     ij = 0
+
+     variable%vt0_field(:,:,ibl) = 0._real64
 
      call kernel(geometry, local_dims, struct, variable)
      call nested_kernel_write(struct%e%p(:,:,local_dims%kbl))
@@ -243,9 +250,10 @@ end subroutine driver
 def fixture_expected_analysis():
     return {
         'local_dims': {
-            'kbl': 'read',
+            'kbl': 'write',
             'kend': 'read',
-            'kst': 'read'
+            'kst': 'read',
+            'ngpblks': 'read'
         },
         'geometry': {
             'dim': {
@@ -255,7 +263,7 @@ def fixture_expected_analysis():
         },
         'array_arg': 'write',
         'variable' : {
-            'vt0_field' : 'read'
+            'vt0_field' : 'write'
         },
         'struct': {
             'a': {
@@ -278,7 +286,8 @@ def fixture_expected_analysis():
                     'p': 'readwrite'
                 }
             }
-        }
+        },
+        'ij': 'write'
     }
 
 
@@ -465,7 +474,7 @@ def check_other_variable_type(mode, conds, pragmas, routine):
             pragmas = FindNodes(ir.Pragma).visit(cond.body)
 
             calls = [call for call in calls
-                     if call.name.name.lower() == 'variable%f_t0%get_device_data_rdonly'
+                     if call.name.name.lower() == 'variable%f_t0%get_device_data_wronly'
                      and 'variable%vt0_field' in call.arguments]
             pragmas = [pragma for pragma in pragmas
                        if 'unstructured-data attach' in pragma.content and 'variable%vt0_field' in pragma.content]
@@ -663,6 +672,8 @@ def test_offload_deepcopy_transformation(frontend, config, deepcopy_code, presen
         # check data present region
         with pragma_regions_attached(driver):
 
+            assert not any('update host' in p.content and '(ij)' in p.content for p in pragmas)
+
             region = FindNodes(ir.PragmaRegion).visit(driver.body)[-1]
             assert is_loki_pragma(region.pragma, starts_with='structured-data')
 
