@@ -25,6 +25,7 @@ from loki.transformations.single_column.revector import (
 )
 from loki.transformations.single_column.vertical import SCCFuseVerticalLoops
 from loki.transformations.pragma_model import PragmaModelTransformation
+from loki.transformations.remove_code import RemoveCodeTransformation
 
 __all__ = [
     'SCCVectorPipeline', 'SCCVVectorPipeline', 'SCCSVectorPipeline',
@@ -35,6 +36,30 @@ __all__ = [
     'SCCRawStackPipeline', 'SCCVRawStackPipeline', 'SCCSRawStackPipeline'
 ]
 
+
+class RemoveUnusedVarTransformation(RemoveCodeTransformation):
+    """
+    A special version of :any:`RemoveCodeTransformation` being a temporary solution
+    that allows to remove unused temporaries/arrays before applying
+    a transformation that handles temporaries on device (hoist, stack)
+
+    The transformation will apply the following methods in order:
+
+    * :any:`do_remove_unused_vars`
+
+    Parameters
+    ----------
+    remove_unused_vars : boolean
+        Remove unused variables/locals from routines.
+    remove_only_arrays : boolean
+        Whether to only remove unused arrays from routines
+        or all variables/locals.
+    """
+    def __init__(self, remove_unused_vars=False, remove_only_arrays=True, **kwargs): # pylint: disable=unused-argument
+        super().__init__(remove_unused_vars=remove_unused_vars, remove_only_arrays=remove_only_arrays,
+                remove_marked_regions=False, kernel_only=True)
+
+
 SCCVVectorPipeline = partial(
     Pipeline, classes=(
         SCCFuseVerticalLoops,
@@ -163,6 +188,7 @@ SCCVHoistPipeline = partial(
         SCCDevectorTransformation,
         SCCDemoteTransformation,
         SCCVecRevectorTransformation,
+        RemoveUnusedVarTransformation,
         HoistTemporaryArraysAnalysis,
         SCCHoistTemporaryArraysTransformation,
         SCCAnnotateTransformation,
@@ -213,6 +239,7 @@ SCCSHoistPipeline = partial(
         SCCDevectorTransformation,
         SCCDemoteTransformation,
         SCCSeqRevectorTransformation,
+        RemoveUnusedVarTransformation,
         HoistTemporaryArraysAnalysis,
         SCCHoistTemporaryArraysTransformation,
         SCCAnnotateTransformation,
@@ -266,6 +293,7 @@ SCCVStackPipeline = partial(
         SCCDevectorTransformation,
         SCCDemoteTransformation,
         SCCVecRevectorTransformation,
+        RemoveUnusedVarTransformation,
         SCCAnnotateTransformation,
         TemporariesPoolAllocatorTransformation,
         PragmaModelTransformation
@@ -315,6 +343,7 @@ SCCSStackPipeline = partial(
         SCCDevectorTransformation,
         SCCDemoteTransformation,
         SCCSeqRevectorTransformation,
+        RemoveUnusedVarTransformation,
         SCCAnnotateTransformation,
         TemporariesPoolAllocatorTransformation,
         PragmaModelTransformation
@@ -361,6 +390,7 @@ SCCVStackFtrPtrPipeline = partial(
         SCCDevectorTransformation,
         SCCDemoteTransformation,
         SCCVecRevectorTransformation,
+        RemoveUnusedVarTransformation,
         SCCAnnotateTransformation,
         FtrPtrStackTransformation,
         PragmaModelTransformation
@@ -410,6 +440,7 @@ SCCSStackFtrPtrPipeline = partial(
         SCCDevectorTransformation,
         SCCDemoteTransformation,
         SCCSeqRevectorTransformation,
+        RemoveUnusedVarTransformation,
         SCCAnnotateTransformation,
         FtrPtrStackTransformation,
         PragmaModelTransformation
@@ -456,6 +487,7 @@ SCCVStackDirectIdxPipeline = partial(
         SCCDevectorTransformation,
         SCCDemoteTransformation,
         SCCVecRevectorTransformation,
+        RemoveUnusedVarTransformation,
         SCCAnnotateTransformation,
         DirectIdxStackTransformation,
         PragmaModelTransformation
@@ -509,6 +541,7 @@ SCCSStackDirectIdxPipeline = partial(
         SCCDevectorTransformation,
         SCCDemoteTransformation,
         SCCSeqRevectorTransformation,
+        RemoveUnusedVarTransformation,
         SCCAnnotateTransformation,
         DirectIdxStackTransformation,
         PragmaModelTransformation
@@ -558,6 +591,7 @@ SCCVRawStackPipeline = partial(
         SCCDevectorTransformation,
         SCCDemoteTransformation,
         SCCVecRevectorTransformation,
+        RemoveUnusedVarTransformation,
         SCCAnnotateTransformation,
         TemporariesRawStackTransformation,
         PragmaModelTransformation
