@@ -23,12 +23,14 @@ from loki.types import BasicType
 from loki.transformations.array_indexing import demote_variables
 from loki.transformations.utilities import (
     get_integer_variable, get_loop_bounds, find_driver_loops,
-    get_local_arrays, check_routine_sequential
+    get_local_arrays, check_routine_sequential,
+    single_variable_declaration
 )
 
 
 __all__ = [
     'SCCDevectorTransformation', 'SCCRevectorTransformation',
+    'SCCVecRevectorTransformation', 'SCCSeqRevectorTransformation',
     'SCCDemoteTransformation', 'wrap_vector_section'
 ]
 
@@ -252,7 +254,7 @@ class SCCDevectorTransformation(Transformation):
         routine.body = Transformer(driver_loop_map).visit(routine.body)
 
 
-def wrap_vector_section(section, routine, horizontal, insert_pragma=True):
+def wrap_vector_section_from_dimension(section, routine, horizontal, insert_pragma=True):
     """
     Wrap a section of nodes in a vector-level loop across the horizontal.
 
@@ -268,9 +270,26 @@ def wrap_vector_section(section, routine, horizontal, insert_pragma=True):
         Adds a ``!$loki vector`` pragma around the created loop
     """
     bounds = get_loop_bounds(routine, dimension=horizontal)
+    return wrap_vector_section(section, routine, bounds, horizontal.index, insert_pragma=insert_pragma)
 
+
+def wrap_vector_section(section, routine, bounds, index, insert_pragma=True):
+    """
+    Wrap a section of nodes in a vector-level loop across the horizontal.
+    
+    Parameters
+    ----------
+    section : tuple of :any:`Node`
+        A section of nodes to be wrapped in a vector-level loop
+    routine : :any:`Subroutine`
+        The subroutine in the vector loops should be removed.
+    horizontal: :any:`Dimension`
+        The dimension specifying the horizontal vector dimension
+    insert_pragma: bool, optional
+        Adds a ``!$loki vector`` pragma around the created loop
+    """
     # Create a single loop around the horizontal from a given body
-    index = get_integer_variable(routine, horizontal.index)
+    index = get_integer_variable(routine, index)
     bounds = sym.LoopRange(bounds)
 
     # Ensure we clone all body nodes, to avoid recursion issues
@@ -285,9 +304,9 @@ def wrap_vector_section(section, routine, horizontal, insert_pragma=True):
     return (ir.Comment(''), vector_loop, ir.Comment(''))
 
 
-class SCCRevectorTransformation(Transformation):
+class BaseRevectorTransformation(Transformation):
     """
-    A transformation to wrap thread-parallel IR sections within a horizontal loop.
+    A base/parent class for transformation to wrap thread-parallel IR sections within a horizontal loop.
     This transformation relies on markers placed by :any:`SCCDevectorTransformation`.
 
     Parameters
@@ -297,9 +316,8 @@ class SCCRevectorTransformation(Transformation):
         to define the horizontal data dimension and iteration space.
     """
 
-    def __init__(self, horizontal, remove_vector_section=False):
+    def __init__(self, horizontal):
         self.horizontal = horizontal
-        self.remove_vector_section = remove_vector_section
 
     def revector_section(self, routine, section):
         """
@@ -317,7 +335,7 @@ class SCCRevectorTransformation(Transformation):
         """
         # Wrap all thread-parallel sections into horizontal thread loops
         mapper = {
-            s: wrap_vector_section(s.body, routine, self.horizontal)
+            s: wrap_vector_section_from_dimension(s.body, routine, self.horizontal)
             for s in FindNodes(ir.Section).visit(section)
             if s.label == 'vector_section'
         }
@@ -347,7 +365,6 @@ class SCCRevectorTransformation(Transformation):
                         loops[0]._update(pragma=(pragma,))
                         region._update(pragma=None, pragma_post=None)
 
-
     def mark_seq_loops(self, section):
         """
         Mark interior sequential loops in a thread-parallel section
@@ -394,6 +411,19 @@ class SCCRevectorTransformation(Transformation):
         )
         loop._update(pragma=loop_pragmas + (pragma,))
 
+
+class SCCVecRevectorTransformation(BaseRevectorTransformation):
+    """
+    A transformation to wrap thread-parallel IR sections within a horizontal loop.
+    This transformation relies on markers placed by :any:`SCCDevectorTransformation`.
+
+    Parameters
+    ----------
+    horizontal : :any:`Dimension`
+        :any:`Dimension` object describing the variable conventions used in code
+        to define the horizontal data dimension and iteration space.
+    """
+
     def transform_subroutine(self, routine, **kwargs):
         """
         Wrap vector-parallel sections in vector :any:`Loop` objects.
@@ -450,6 +480,195 @@ class SCCRevectorTransformation(Transformation):
                     # Mark outer driver loops
                     self.mark_driver_loop(routine, loop)
 
+# alias for backwards compability
+SCCRevectorTransformation = SCCVecRevectorTransformation
+
+class SCCSeqRevectorTransformation(BaseRevectorTransformation):
+    """
+    A transformation to wrap thread-parallel IR sections within a horizontal loop
+    in a way that the horizontal loop is hoisted/moved to the driver level while
+    the horizontal/loop index is passed as an argument.
+    This transformation relies on markers placed by :any:`SCCDevectorTransformation`.
+    Parameters
+    ----------
+    horizontal : :any:`Dimension`
+        :any:`Dimension` object describing the variable conventions used in code
+        to define the horizontal data dimension and iteration space.
+    """
+
+    process_ignored_items = True
+
+    def remove_vector_sections(self, section):
+        """
+        Remove all thread-parallel :any:`Section` objects within a given
+        code section
+        Parameters
+        ----------
+        routine : :any:`Subroutine`
+            Subroutine to apply this transformation to.
+        section : tuple of :any:`Node`
+            Code section in which to replace vector-parallel
+            :any:`Section` objects.
+        """
+        # Wrap all thread-parallel sections into horizontal thread loops
+        mapper = {
+            s: s.body
+            for s in FindNodes(ir.Section).visit(section)
+            if s.label == 'vector_section'
+        }
+        return Transformer(mapper).visit(section)
+
+    def mark_vector_reductions(self, routine, section):
+        """
+        Mark vector-reduction loops in marked vector-reduction
+        regions.
+        If a region explicitly marked with
+        ``!$loki vector-reduction(<reduction clause>)``/
+        ``!$loki end vector-reduction`` is encountered, we replace
+        existing ``!$loki loop vector`` loop pragmas and add the
+        reduction keyword and clause. These will be turned into
+        OpenACC equivalents by :any:`SCCAnnotate`.
+        """
+        with pragma_regions_attached(routine):
+            for region in FindNodes(ir.PragmaRegion).visit(section):
+                if is_loki_pragma(region.pragma, starts_with='vector-reduction'):
+                    if (reduction_clause := re.search(r'reduction\([\w:0-9 \t]+\)', region.pragma.content)):
+
+                        loops = FindNodes(ir.Loop).visit(region)
+                        if loops:
+                            pragma = ir.Pragma(keyword='loki', content=f'loop vector {reduction_clause[0]}')
+                            # Update loop and region in place to remove marker pragmas
+                            loops[0]._update(pragma=(pragma,))
+                            region._update(pragma=None, pragma_post=None)
+
+    @staticmethod
+    def _get_loop_bound(bound, call_arg_map):
+        if isinstance(bound, tuple):
+            for alias in bound:
+                alias_arg = alias
+                elem = None
+                if '%' in alias:
+                    elem = alias_arg.split('%')[1]
+                    alias_arg = alias_arg.split('%')[0]
+                if alias_arg in call_arg_map:
+                    return (call_arg_map[alias_arg.lower()], elem)
+        return (call_arg_map[bound.lower()], None)
+
+    def transform_subroutine(self, routine, **kwargs):
+        """
+        Wrap vector-parallel sections in vector :any:`Loop` objects.
+        This wraps all thread-parallel sections within "kernel"
+        routines or within the parallel loops in "driver" routines.
+        The markers placed by :any:`SCCDevectorTransformation` are removed
+        Parameters
+        ----------
+        routine : :any:`Subroutine`
+            Subroutine to apply this transformation to.
+        role : str
+            Must be either ``"kernel"`` or ``"driver"``
+        targets : tuple or str
+            Tuple of target routine names for determining "driver" loops
+        """
+        role = kwargs['role']
+        targets = tuple(str(t).lower() for t in as_tuple(kwargs.get('targets', None)))
+        # ignore = kwargs.get('ignore', ())
+        item = kwargs.get('item', None)
+        ignore = item.ignore if item else () + tuple(str(t).lower() for t in as_tuple(kwargs.get('ignore', None)))
+
+        if role == 'kernel':
+            # Skip if kernel is marked as `!$loki routine seq`
+            if check_routine_sequential(routine):
+                return
+
+            if self.horizontal.index not in routine.variables:
+                jl = get_integer_variable(routine, self.horizontal.index)
+                routine.arguments += (jl.clone(type=jl.type.clone(intent='in')),)
+            else:
+                single_variable_declaration(routine, variables=(self.horizontal.index,))
+                routine.symbol_attrs.update({self.horizontal.index:\
+                    routine.variable_map[self.horizontal.index].type.clone(intent='in')})
+                if self.horizontal.index not in routine.arguments:
+                    routine.arguments += (get_integer_variable(routine, self.horizontal.index),)
+
+            # add horizontal.index as argument for calls/routines being in targets
+            call_map = {}
+            for call in FindNodes(ir.CallStatement).visit(routine.body):
+                if str(call.name).lower() in targets or call.routine.name.lower() in ignore:
+                    if check_routine_sequential(call.routine):
+                        continue
+                    if self.horizontal.index not in call.arg_map:
+                        new_kwarg = (self.horizontal.index, get_integer_variable(routine, self.horizontal.index))
+                        updated_call = call.clone(kwarguments=call.kwarguments + (new_kwarg,))
+                        call_map[call] = updated_call
+                    if call.routine.name.lower() in ignore:
+                        if self.horizontal.index not in call.routine.variables:
+                            jl = get_integer_variable(call.routine, self.horizontal.index)
+                            call.routine.arguments += (jl.clone(type=jl.type.clone(intent='in')),)
+                        else:
+                            single_variable_declaration(call.routine, variables=(self.horizontal.index,))
+                            call.routine.symbol_attrs.update({self.horizontal.index:\
+                                call.routine.variable_map[self.horizontal.index].type.clone(intent='in')})
+                            if self.horizontal.index not in call.routine.arguments:
+                                call.routine.arguments += (get_integer_variable(call.routine, self.horizontal.index),)
+            routine.body = Transformer(call_map).visit(routine.body)
+
+            # Revector all marked vector sections within the kernel body
+            routine.body = self.remove_vector_sections(routine.body)
+
+            with pragmas_attached(routine, ir.Loop):
+                # Check for explicitly labelled vector-reduction regions
+                self.mark_vector_reductions(routine, routine.body)
+
+                # Mark sequential loops inside vector sections
+                self.mark_seq_loops(routine.body)
+
+            # Mark subroutine as seq for later annotation
+            routine.spec.append(ir.Pragma(keyword='loki', content='routine seq'))
+
+        if role == 'driver':
+
+            # add horizontal.index, e.g., 'jl'
+            index = get_integer_variable(routine, self.horizontal.index)
+            routine.variables += (index,)
+
+            with pragmas_attached(routine, ir.Loop):
+                driver_loops = find_driver_loops(section=routine.body, targets=targets)
+
+                for loop in driver_loops:
+
+                    # Wrap calls being in targets in a horizontal loop and add horizontal.index as argument
+                    call_map = {}
+                    for call in FindNodes(ir.CallStatement).visit(loop.body):
+                        if str(call.name).lower() in targets:
+                            if self.horizontal.index not in call.arg_map:
+                                new_kwarg = (self.horizontal.index,
+                                        get_integer_variable(routine, self.horizontal.index))
+                                updated_call = call.clone(kwarguments=call.kwarguments + (new_kwarg,))
+                                call_arg_map = {k.name.lower(): v for (k, v) in call.arg_map.items()}
+                                # loop bound(s) could be derived types ...
+                                ltmp = self._get_loop_bound(self.horizontal.lower, call_arg_map)
+                                utmp = self._get_loop_bound(self.horizontal.upper, call_arg_map)
+                                lower = ltmp[0] if ltmp[1] is None else sym.Variable(name=ltmp[1], parent=ltmp[0])
+                                upper = utmp[0] if utmp[1] is None else sym.Variable(name=utmp[1], parent=utmp[0])
+                                # wrap call with horizontal loop
+                                loop_bounds = (lower, upper)
+                                call_map[call] = wrap_vector_section((updated_call,), routine, bounds=loop_bounds,
+                                        insert_pragma=True, index=self.horizontal.index)
+
+                    loop._update(body=Transformer(call_map).visit(loop.body))
+
+                    # Revector all marked sections within the driver loop body
+                    loop._update(body=self.revector_section(routine, loop.body))
+
+                    # Check for explicitly labelled vector-reduction regions
+                    super().mark_vector_reductions(routine, loop.body)
+
+                    # Mark sequential loops inside vector sections
+                    self.mark_seq_loops(loop.body)
+
+                    # Mark outer driver loops
+                    self.mark_driver_loop(routine, loop)
+
 
 class SCCDemoteTransformation(Transformation):
     """
