@@ -89,6 +89,8 @@ from loki.ir import (
     Assignment, FindVariables, FindInlineCalls, SubstituteExpressions
 )
 from loki.tools.util import is_iterable, as_tuple, CaseInsensitiveDict, flatten
+from loki.types import BasicType
+from loki.logging import warning
 
 from loki.transformations.utilities import single_variable_declaration
 
@@ -163,6 +165,14 @@ class HoistVariablesAnalysis(Transformation):
             if not isinstance(child, ProcedureItem):
                 continue
 
+            if call_map[child.local_name].routine is BasicType.DEFERRED:
+                warning((
+                    '[Loki::HoistVariablesAnalysis] '
+                    f''
+                    f'call.routine is BasicType.DEFERRED for call to {child.local_name} in {routine.name}'
+                ))
+                continue
+
             # We may call a subroutine again with aliased sizes, so we check hoisted
             # variables in children by name before adding them
             hoist_var_names = [v.name.lower() for v in item.trafo_data[self._key]["hoist_variables"]]
@@ -280,6 +290,14 @@ class HoistVariablesTransformation(Transformation):
             if str(call.name) not in successor_map:
                 continue
 
+            if call.routine is BasicType.DEFERRED:
+                warning((
+                    '[Loki::HoistVariablesTransformation] '
+                    f''
+                    f'call.routine is BasicType.DEFERRED for call to {call.name} in {routine.name}'
+                ))
+                continue
+
             successor_item = successor_map[str(call.routine.name)]
             if self.as_kwarguments:
                 to_hoist = successor_item.trafo_data[self._key]["to_hoist"]
